@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -17,5 +17,9 @@ namespace Ip
 {
 class Address;
 }
+
+typedef uint32_t nfmark_t;
+typedef unsigned char tos_t;
+
 #endif /* _SQUID_IP_FORWARD_H */
 
